# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreateOceInstanceDetails(object):
    """
    The information about new OceInstance.
    """

    #: A constant which can be used with the instance_usage_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "PRIMARY"
    INSTANCE_USAGE_TYPE_PRIMARY = "PRIMARY"

    #: A constant which can be used with the instance_usage_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "NONPRIMARY"
    INSTANCE_USAGE_TYPE_NONPRIMARY = "NONPRIMARY"

    #: A constant which can be used with the instance_access_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "PUBLIC"
    INSTANCE_ACCESS_TYPE_PUBLIC = "PUBLIC"

    #: A constant which can be used with the instance_access_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "PRIVATE"
    INSTANCE_ACCESS_TYPE_PRIVATE = "PRIVATE"

    #: A constant which can be used with the instance_license_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "NEW"
    INSTANCE_LICENSE_TYPE_NEW = "NEW"

    #: A constant which can be used with the instance_license_type property of a CreateOceInstanceDetails.
    #: This constant has a value of "BYOL"
    INSTANCE_LICENSE_TYPE_BYOL = "BYOL"

    def __init__(self, **kwargs):
        """
        Initializes a new CreateOceInstanceDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param description:
            The value to assign to the description property of this CreateOceInstanceDetails.
        :type description: str

        :param compartment_id:
            The value to assign to the compartment_id property of this CreateOceInstanceDetails.
        :type compartment_id: str

        :param name:
            The value to assign to the name property of this CreateOceInstanceDetails.
        :type name: str

        :param tenancy_id:
            The value to assign to the tenancy_id property of this CreateOceInstanceDetails.
        :type tenancy_id: str

        :param idcs_access_token:
            The value to assign to the idcs_access_token property of this CreateOceInstanceDetails.
        :type idcs_access_token: str

        :param identity_stripe:
            The value to assign to the identity_stripe property of this CreateOceInstanceDetails.
        :type identity_stripe: IdentityStripeDetails

        :param tenancy_name:
            The value to assign to the tenancy_name property of this CreateOceInstanceDetails.
        :type tenancy_name: str

        :param instance_usage_type:
            The value to assign to the instance_usage_type property of this CreateOceInstanceDetails.
            Allowed values for this property are: "PRIMARY", "NONPRIMARY"
        :type instance_usage_type: str

        :param object_storage_namespace:
            The value to assign to the object_storage_namespace property of this CreateOceInstanceDetails.
        :type object_storage_namespace: str

        :param admin_email:
            The value to assign to the admin_email property of this CreateOceInstanceDetails.
        :type admin_email: str

        :param upgrade_schedule:
            The value to assign to the upgrade_schedule property of this CreateOceInstanceDetails.
        :type upgrade_schedule: str

        :param waf_primary_domain:
            The value to assign to the waf_primary_domain property of this CreateOceInstanceDetails.
        :type waf_primary_domain: str

        :param instance_access_type:
            The value to assign to the instance_access_type property of this CreateOceInstanceDetails.
            Allowed values for this property are: "PUBLIC", "PRIVATE"
        :type instance_access_type: str

        :param instance_license_type:
            The value to assign to the instance_license_type property of this CreateOceInstanceDetails.
            Allowed values for this property are: "NEW", "BYOL"
        :type instance_license_type: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreateOceInstanceDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this CreateOceInstanceDetails.
        :type defined_tags: dict(str, dict(str, object))

        """
        self.swagger_types = {
            'description': 'str',
            'compartment_id': 'str',
            'name': 'str',
            'tenancy_id': 'str',
            'idcs_access_token': 'str',
            'identity_stripe': 'IdentityStripeDetails',
            'tenancy_name': 'str',
            'instance_usage_type': 'str',
            'object_storage_namespace': 'str',
            'admin_email': 'str',
            'upgrade_schedule': 'str',
            'waf_primary_domain': 'str',
            'instance_access_type': 'str',
            'instance_license_type': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))'
        }

        self.attribute_map = {
            'description': 'description',
            'compartment_id': 'compartmentId',
            'name': 'name',
            'tenancy_id': 'tenancyId',
            'idcs_access_token': 'idcsAccessToken',
            'identity_stripe': 'identityStripe',
            'tenancy_name': 'tenancyName',
            'instance_usage_type': 'instanceUsageType',
            'object_storage_namespace': 'objectStorageNamespace',
            'admin_email': 'adminEmail',
            'upgrade_schedule': 'upgradeSchedule',
            'waf_primary_domain': 'wafPrimaryDomain',
            'instance_access_type': 'instanceAccessType',
            'instance_license_type': 'instanceLicenseType',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags'
        }

        self._description = None
        self._compartment_id = None
        self._name = None
        self._tenancy_id = None
        self._idcs_access_token = None
        self._identity_stripe = None
        self._tenancy_name = None
        self._instance_usage_type = None
        self._object_storage_namespace = None
        self._admin_email = None
        self._upgrade_schedule = None
        self._waf_primary_domain = None
        self._instance_access_type = None
        self._instance_license_type = None
        self._freeform_tags = None
        self._defined_tags = None

    @property
    def description(self):
        """
        Gets the description of this CreateOceInstanceDetails.
        OceInstance description


        :return: The description of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this CreateOceInstanceDetails.
        OceInstance description


        :param description: The description of this CreateOceInstanceDetails.
        :type: str
        """
        self._description = description

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this CreateOceInstanceDetails.
        Compartment Identifier


        :return: The compartment_id of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this CreateOceInstanceDetails.
        Compartment Identifier


        :param compartment_id: The compartment_id of this CreateOceInstanceDetails.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def name(self):
        """
        **[Required]** Gets the name of this CreateOceInstanceDetails.
        OceInstance Name


        :return: The name of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this CreateOceInstanceDetails.
        OceInstance Name


        :param name: The name of this CreateOceInstanceDetails.
        :type: str
        """
        self._name = name

    @property
    def tenancy_id(self):
        """
        **[Required]** Gets the tenancy_id of this CreateOceInstanceDetails.
        Tenancy Identifier


        :return: The tenancy_id of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._tenancy_id

    @tenancy_id.setter
    def tenancy_id(self, tenancy_id):
        """
        Sets the tenancy_id of this CreateOceInstanceDetails.
        Tenancy Identifier


        :param tenancy_id: The tenancy_id of this CreateOceInstanceDetails.
        :type: str
        """
        self._tenancy_id = tenancy_id

    @property
    def idcs_access_token(self):
        """
        **[Required]** Gets the idcs_access_token of this CreateOceInstanceDetails.
        Identity Cloud Service access token identifying a stripe and service administrator user


        :return: The idcs_access_token of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._idcs_access_token

    @idcs_access_token.setter
    def idcs_access_token(self, idcs_access_token):
        """
        Sets the idcs_access_token of this CreateOceInstanceDetails.
        Identity Cloud Service access token identifying a stripe and service administrator user


        :param idcs_access_token: The idcs_access_token of this CreateOceInstanceDetails.
        :type: str
        """
        self._idcs_access_token = idcs_access_token

    @property
    def identity_stripe(self):
        """
        Gets the identity_stripe of this CreateOceInstanceDetails.

        :return: The identity_stripe of this CreateOceInstanceDetails.
        :rtype: IdentityStripeDetails
        """
        return self._identity_stripe

    @identity_stripe.setter
    def identity_stripe(self, identity_stripe):
        """
        Sets the identity_stripe of this CreateOceInstanceDetails.

        :param identity_stripe: The identity_stripe of this CreateOceInstanceDetails.
        :type: IdentityStripeDetails
        """
        self._identity_stripe = identity_stripe

    @property
    def tenancy_name(self):
        """
        **[Required]** Gets the tenancy_name of this CreateOceInstanceDetails.
        Tenancy Name


        :return: The tenancy_name of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._tenancy_name

    @tenancy_name.setter
    def tenancy_name(self, tenancy_name):
        """
        Sets the tenancy_name of this CreateOceInstanceDetails.
        Tenancy Name


        :param tenancy_name: The tenancy_name of this CreateOceInstanceDetails.
        :type: str
        """
        self._tenancy_name = tenancy_name

    @property
    def instance_usage_type(self):
        """
        Gets the instance_usage_type of this CreateOceInstanceDetails.
        Instance type based on its usage

        Allowed values for this property are: "PRIMARY", "NONPRIMARY"


        :return: The instance_usage_type of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._instance_usage_type

    @instance_usage_type.setter
    def instance_usage_type(self, instance_usage_type):
        """
        Sets the instance_usage_type of this CreateOceInstanceDetails.
        Instance type based on its usage


        :param instance_usage_type: The instance_usage_type of this CreateOceInstanceDetails.
        :type: str
        """
        allowed_values = ["PRIMARY", "NONPRIMARY"]
        if not value_allowed_none_or_none_sentinel(instance_usage_type, allowed_values):
            raise ValueError(
                "Invalid value for `instance_usage_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._instance_usage_type = instance_usage_type

    @property
    def object_storage_namespace(self):
        """
        **[Required]** Gets the object_storage_namespace of this CreateOceInstanceDetails.
        Object Storage Namespace of Tenancy


        :return: The object_storage_namespace of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._object_storage_namespace

    @object_storage_namespace.setter
    def object_storage_namespace(self, object_storage_namespace):
        """
        Sets the object_storage_namespace of this CreateOceInstanceDetails.
        Object Storage Namespace of Tenancy


        :param object_storage_namespace: The object_storage_namespace of this CreateOceInstanceDetails.
        :type: str
        """
        self._object_storage_namespace = object_storage_namespace

    @property
    def admin_email(self):
        """
        **[Required]** Gets the admin_email of this CreateOceInstanceDetails.
        Admin Email for Notification


        :return: The admin_email of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._admin_email

    @admin_email.setter
    def admin_email(self, admin_email):
        """
        Sets the admin_email of this CreateOceInstanceDetails.
        Admin Email for Notification


        :param admin_email: The admin_email of this CreateOceInstanceDetails.
        :type: str
        """
        self._admin_email = admin_email

    @property
    def upgrade_schedule(self):
        """
        Gets the upgrade_schedule of this CreateOceInstanceDetails.
        Upgrade schedule type representing service to be upgraded immediately whenever latest version is released
        or delay upgrade of the service to previous released version


        :return: The upgrade_schedule of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._upgrade_schedule

    @upgrade_schedule.setter
    def upgrade_schedule(self, upgrade_schedule):
        """
        Sets the upgrade_schedule of this CreateOceInstanceDetails.
        Upgrade schedule type representing service to be upgraded immediately whenever latest version is released
        or delay upgrade of the service to previous released version


        :param upgrade_schedule: The upgrade_schedule of this CreateOceInstanceDetails.
        :type: str
        """
        self._upgrade_schedule = upgrade_schedule

    @property
    def waf_primary_domain(self):
        """
        Gets the waf_primary_domain of this CreateOceInstanceDetails.
        Web Application Firewall(WAF) primary domain


        :return: The waf_primary_domain of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._waf_primary_domain

    @waf_primary_domain.setter
    def waf_primary_domain(self, waf_primary_domain):
        """
        Sets the waf_primary_domain of this CreateOceInstanceDetails.
        Web Application Firewall(WAF) primary domain


        :param waf_primary_domain: The waf_primary_domain of this CreateOceInstanceDetails.
        :type: str
        """
        self._waf_primary_domain = waf_primary_domain

    @property
    def instance_access_type(self):
        """
        Gets the instance_access_type of this CreateOceInstanceDetails.
        Flag indicating whether the instance access is private or public

        Allowed values for this property are: "PUBLIC", "PRIVATE"


        :return: The instance_access_type of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._instance_access_type

    @instance_access_type.setter
    def instance_access_type(self, instance_access_type):
        """
        Sets the instance_access_type of this CreateOceInstanceDetails.
        Flag indicating whether the instance access is private or public


        :param instance_access_type: The instance_access_type of this CreateOceInstanceDetails.
        :type: str
        """
        allowed_values = ["PUBLIC", "PRIVATE"]
        if not value_allowed_none_or_none_sentinel(instance_access_type, allowed_values):
            raise ValueError(
                "Invalid value for `instance_access_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._instance_access_type = instance_access_type

    @property
    def instance_license_type(self):
        """
        Gets the instance_license_type of this CreateOceInstanceDetails.
        Flag indicating whether the instance license is new cloud or bring your own license

        Allowed values for this property are: "NEW", "BYOL"


        :return: The instance_license_type of this CreateOceInstanceDetails.
        :rtype: str
        """
        return self._instance_license_type

    @instance_license_type.setter
    def instance_license_type(self, instance_license_type):
        """
        Sets the instance_license_type of this CreateOceInstanceDetails.
        Flag indicating whether the instance license is new cloud or bring your own license


        :param instance_license_type: The instance_license_type of this CreateOceInstanceDetails.
        :type: str
        """
        allowed_values = ["NEW", "BYOL"]
        if not value_allowed_none_or_none_sentinel(instance_license_type, allowed_values):
            raise ValueError(
                "Invalid value for `instance_license_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._instance_license_type = instance_license_type

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this CreateOceInstanceDetails.
        Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
        Example: `{\"bar-key\": \"value\"}`


        :return: The freeform_tags of this CreateOceInstanceDetails.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this CreateOceInstanceDetails.
        Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
        Example: `{\"bar-key\": \"value\"}`


        :param freeform_tags: The freeform_tags of this CreateOceInstanceDetails.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this CreateOceInstanceDetails.
        Usage of predefined tag keys. These predefined keys are scoped to namespaces.
        Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`


        :return: The defined_tags of this CreateOceInstanceDetails.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this CreateOceInstanceDetails.
        Usage of predefined tag keys. These predefined keys are scoped to namespaces.
        Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`


        :param defined_tags: The defined_tags of this CreateOceInstanceDetails.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from __future__ import absolute_import

from .backup_key_details import BackupKeyDetails
from .backup_location import BackupLocation
from .backup_location_bucket import BackupLocationBucket
from .backup_location_uri import BackupLocationURI
from .backup_vault_details import BackupVaultDetails
from .change_key_compartment_details import ChangeKeyCompartmentDetails
from .change_vault_compartment_details import ChangeVaultCompartmentDetails
from .create_key_details import CreateKeyDetails
from .create_vault_details import CreateVaultDetails
from .decrypt_data_details import DecryptDataDetails
from .decrypted_data import DecryptedData
from .encrypt_data_details import EncryptDataDetails
from .encrypted_data import EncryptedData
from .export_key_details import ExportKeyDetails
from .exported_key_data import ExportedKeyData
from .generate_key_details import GenerateKeyDetails
from .generated_key import GeneratedKey
from .import_key_details import ImportKeyDetails
from .import_key_version_details import ImportKeyVersionDetails
from .key import Key
from .key_shape import KeyShape
from .key_summary import KeySummary
from .key_version import KeyVersion
from .key_version_summary import KeyVersionSummary
from .restore_key_from_object_store_details import RestoreKeyFromObjectStoreDetails
from .restore_vault_from_object_store_details import RestoreVaultFromObjectStoreDetails
from .schedule_key_deletion_details import ScheduleKeyDeletionDetails
from .schedule_key_version_deletion_details import ScheduleKeyVersionDeletionDetails
from .schedule_vault_deletion_details import ScheduleVaultDeletionDetails
from .update_key_details import UpdateKeyDetails
from .update_vault_details import UpdateVaultDetails
from .vault import Vault
from .vault_summary import VaultSummary
from .vault_usage import VaultUsage
from .wrapped_import_key import WrappedImportKey
from .wrapping_key import WrappingKey

# Maps type names to classes for key_management services.
key_management_type_mapping = {
    "BackupKeyDetails": BackupKeyDetails,
    "BackupLocation": BackupLocation,
    "BackupLocationBucket": BackupLocationBucket,
    "BackupLocationURI": BackupLocationURI,
    "BackupVaultDetails": BackupVaultDetails,
    "ChangeKeyCompartmentDetails": ChangeKeyCompartmentDetails,
    "ChangeVaultCompartmentDetails": ChangeVaultCompartmentDetails,
    "CreateKeyDetails": CreateKeyDetails,
    "CreateVaultDetails": CreateVaultDetails,
    "DecryptDataDetails": DecryptDataDetails,
    "DecryptedData": DecryptedData,
    "EncryptDataDetails": EncryptDataDetails,
    "EncryptedData": EncryptedData,
    "ExportKeyDetails": ExportKeyDetails,
    "ExportedKeyData": ExportedKeyData,
    "GenerateKeyDetails": GenerateKeyDetails,
    "GeneratedKey": GeneratedKey,
    "ImportKeyDetails": ImportKeyDetails,
    "ImportKeyVersionDetails": ImportKeyVersionDetails,
    "Key": Key,
    "KeyShape": KeyShape,
    "KeySummary": KeySummary,
    "KeyVersion": KeyVersion,
    "KeyVersionSummary": KeyVersionSummary,
    "RestoreKeyFromObjectStoreDetails": RestoreKeyFromObjectStoreDetails,
    "RestoreVaultFromObjectStoreDetails": RestoreVaultFromObjectStoreDetails,
    "ScheduleKeyDeletionDetails": ScheduleKeyDeletionDetails,
    "ScheduleKeyVersionDeletionDetails": ScheduleKeyVersionDeletionDetails,
    "ScheduleVaultDeletionDetails": ScheduleVaultDeletionDetails,
    "UpdateKeyDetails": UpdateKeyDetails,
    "UpdateVaultDetails": UpdateVaultDetails,
    "Vault": Vault,
    "VaultSummary": VaultSummary,
    "VaultUsage": VaultUsage,
    "WrappedImportKey": WrappedImportKey,
    "WrappingKey": WrappingKey
}



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .create_db_home_base import CreateDbHomeBase
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreateDbHomeWithDbSystemIdFromDatabaseDetails(CreateDbHomeBase):
    """
    Note that a valid `dbSystemId` value must be supplied for the `CreateDbHomeWithDbSystemIdFromDatabase` API operation to successfully complete.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new CreateDbHomeWithDbSystemIdFromDatabaseDetails object with values from keyword arguments. The default value of the :py:attr:`~oci.database.models.CreateDbHomeWithDbSystemIdFromDatabaseDetails.source` attribute
        of this class is ``DATABASE`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param display_name:
            The value to assign to the display_name property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type display_name: str

        :param database_software_image_id:
            The value to assign to the database_software_image_id property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type database_software_image_id: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type defined_tags: dict(str, dict(str, object))

        :param source:
            The value to assign to the source property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
            Allowed values for this property are: "NONE", "DB_BACKUP", "DATABASE", "VM_CLUSTER_NEW"
        :type source: str

        :param db_system_id:
            The value to assign to the db_system_id property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type db_system_id: str

        :param database:
            The value to assign to the database property of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type database: CreateDatabaseFromAnotherDatabaseDetails

        """
        self.swagger_types = {
            'display_name': 'str',
            'database_software_image_id': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))',
            'source': 'str',
            'db_system_id': 'str',
            'database': 'CreateDatabaseFromAnotherDatabaseDetails'
        }

        self.attribute_map = {
            'display_name': 'displayName',
            'database_software_image_id': 'databaseSoftwareImageId',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags',
            'source': 'source',
            'db_system_id': 'dbSystemId',
            'database': 'database'
        }

        self._display_name = None
        self._database_software_image_id = None
        self._freeform_tags = None
        self._defined_tags = None
        self._source = None
        self._db_system_id = None
        self._database = None
        self._source = 'DATABASE'

    @property
    def db_system_id(self):
        """
        **[Required]** Gets the db_system_id of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        The `OCID`__ of the DB system.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The db_system_id of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :rtype: str
        """
        return self._db_system_id

    @db_system_id.setter
    def db_system_id(self, db_system_id):
        """
        Sets the db_system_id of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        The `OCID`__ of the DB system.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param db_system_id: The db_system_id of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type: str
        """
        self._db_system_id = db_system_id

    @property
    def database(self):
        """
        **[Required]** Gets the database of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.

        :return: The database of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :rtype: CreateDatabaseFromAnotherDatabaseDetails
        """
        return self._database

    @database.setter
    def database(self, database):
        """
        Sets the database of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.

        :param database: The database of this CreateDbHomeWithDbSystemIdFromDatabaseDetails.
        :type: CreateDatabaseFromAnotherDatabaseDetails
        """
        self._database = database

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

import io
import hashlib
import base64
from . import md5 as MD5
from multiprocessing.dummy import Pool
from os import stat
from ..constants import DEFAULT_PART_SIZE
from ..constants import MEBIBYTE
from .buffered_part_reader import BufferedPartReader
from ... import models
from ....exceptions import ServiceError, MultipartUploadError
from oci.exceptions import RequestException, ConnectTimeout
from oci._vendor.requests.exceptions import Timeout, ConnectionError
from oci._vendor.six.moves.queue import Queue
from threading import Semaphore
from oci._vendor import six
from oci.fips import is_fips_mode

READ_BUFFER_SIZE = 8 * 1024
DEFAULT_PARALLEL_PROCESS_COUNT = 3
DEFAULT_MAX_RETRIES = 3
SSEC_PARAM_NAMES = [
    'opc_sse_customer_algorithm',
    'opc_sse_customer_key',
    'opc_sse_customer_key_sha256'
]

# This timeout value will be used only for multipart upload operations, since we're running into SSL issues with
# sending memoryviews with a timeout.
MULTIPART_UPLOAD_TIMEOUT = None


class MultipartObjectAssembler:
    def __init__(self,
                 object_storage_client,
                 namespace_name,
                 bucket_name,
                 object_name,
                 **kwargs
                 ):
        """
        MultipartObjectAssembler provides a simplified interaction when uploading large
        objects using multi-part uploads.

        An assembler can be used to begin a new upload, or resume a previous one.

        PLEASE NOTE that the timeout for the object storage client is overwritten to `None` for all operations which
        call object storage. For this reason, the operations are NOT thread-safe, and you should provide the
        MultipartObjectAssembler class with its own Object Storage client that isn't used elsewhere.
        For more information please see `Known Issues <https://oracle-cloud-infrastructure-python-sdk.readthedocs.io/en/latest/known-issues.html>`_

        :param ObjectStorageClient object_storage_client:
            A configured Object Storage client.

        :param str namespace_name:
            The namespace containing the bucket in which to store the object.

        :param str bucket_name:
            The name of the bucket in which to store the object.

        :param str object_name:
            The name to use for the object in Object Storage, or existing object name if resuming.

        :param int part_size (optional):
            Override the default part size of 128 MiB, value is in bytes.

        :param str if_match (optional):
            The entity tag of the object to match.

        :param str if_none_match (optional):
            The entity tag of the object to avoid matching. The only valid value is ‘*’,
            which indicates that the request should fail if the object
             already exists.

        :param str content_type (optional):
            The content type of the object to upload.

        :param str content_language (optional):
            The content language of the object to upload.

        :param str content_encoding (optional):
            The content encoding of the object to upload.

        :param dict metadata (optional):
            A dictionary of string to string values to associate with the object to upload

        :param bool allow_parallel_uploads (optional):
            Whether or not this MultipartObjectAssembler supports uploading parts in parallel. Defaults to True.

        :param int parallel_process_count (optional):
            The number of worker processes to use in parallel for performing a multipart upload. Default is 3.

        :param str opc_sse_customer_algorithm (optional):
            The encryption algorithm to use with the customer provided encryption key.

        :param str opc_sse_customer_key (optional):
            The base64-encoded 256-bit encryption key to use to encrypt the objects uploaded by this MultipartObjectAssembler.

        :param str opc_sse_customer_key_sha256 (optional):
            The base64-encoded SHA256 hash of the encryption key.
        """
        self.object_storage_client = object_storage_client

        self.part_size = DEFAULT_PART_SIZE
        if 'part_size' in kwargs:
            self.part_size = kwargs['part_size']
        self.if_match = None
        if 'if_match' in kwargs:
            self.if_match = kwargs['if_match']
        self.if_none_match = None
        if 'if_none_match' in kwargs:
            self.if_none_match = kwargs['if_none_match']
        self.content_type = None
        if 'content_type' in kwargs:
            self.content_type = kwargs['content_type']
        self.content_language = None
        if 'content_language' in kwargs:
            self.content_language = kwargs['content_language']
        self.content_encoding = None
        if 'content_encoding' in kwargs:
            self.content_encoding = kwargs['content_encoding']
        self.metadata = None
        if 'metadata' in kwargs:
            self.metadata = kwargs['metadata']

        self.parallel_process_count = DEFAULT_PARALLEL_PROCESS_COUNT
        if 'parallel_process_count' in kwargs:
            if kwargs['parallel_process_count'] == 0:
                raise ValueError('parallel_process_count must be at least one.')

            self.parallel_process_count = kwargs['parallel_process_count']

        if 'allow_parallel_uploads' in kwargs and kwargs['allow_parallel_uploads'] is False:
            # if parallel uploads are disabled, use only a single process
            self.parallel_process_count = 1

        self.max_retries = DEFAULT_MAX_RETRIES
        self.manifest = {"uploadId": None,
                         "namespace": namespace_name,
                         "bucketName": bucket_name,
                         "objectName": object_name,
                         "parts": []}
        # Copy SSE-C parameters (if any)
        self.ssec_params = {}
        for param_name in SSEC_PARAM_NAMES:
            if param_name in kwargs:
                self.ssec_params[param_name] = kwargs[param_name]

        # Store the original timeout value
        self._object_storage_client_timeout = self.object_storage_client.base_client.timeout

    @staticmethod
    def calculate_md5(file_path, offset, chunk):
        """
        Calculate the base64 encoded MD5 hash for a part of a file.

        :param str file_path: Path to the file
        :param int offset: Offset where the part starts in the file
        :param int chunk: Number of bytes in the part
        :return: Base64 encoded MD5 hash
        :rtype: str
        """

        # Determine if we can use the hashlib version of md5 or the bundled
        # version of md5
        if is_fips_mode():
            md5 = MD5.md5
        else:
            md5 = hashlib.md5

        m = md5()
        with io.open(file_path, mode='rb') as f:
            bpr = BufferedPartReader(f, offset, chunk)
            while True:
                part = bpr.read(READ_BUFFER_SIZE)
                if part == b'':
                    break
                m.update(part)
        return base64.b64encode(m.digest()).decode("utf-8")

    @staticmethod
    def _is_exception_retryable(e):
        """
        Determines if the service should attempt to retry an operation based
         on the type of exception

        retry if
           timeout
           Connection error
           unknown client exception: status == -1
           server exception: status >= 500
           Potential edge case: status == 409

        :param e: Exception
        :return: Boolean
        """
        retryable = False
        if isinstance(e, Timeout):
            retryable = True
        elif isinstance(e, ConnectionError):
            retryable = True
        elif isinstance(e, RequestException):
            retryable = True
        elif isinstance(e, ConnectTimeout):
            retryable = True
        elif isinstance(e, ServiceError):
            if e.status >= 500 or e.status == -1 or (e.status == 409 and e.code == "ConcurrentObjectUpdate") or (e.status == 429):
                retryable = True

        return retryable

    def _upload_part_call(self, object_storage_client, **kwargs):
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()
            with io.open(kwargs["part_file_path"], mode='rb') as file_object:
                bpr = BufferedPartReader(file_object, kwargs["offset"], kwargs["size"])

                return object_storage_client.upload_part(
                    kwargs["namespace"],
                    kwargs["bucket_name"],
                    kwargs["object_name"],
                    kwargs["upload_id"],
                    kwargs["part_num"],
                    bpr,
                    **kwargs['new_kwargs'])

        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def add_parts_from_file(self, file_path):
        """
        Splits a file into parts and adds all parts to an internal list of parts to upload.  The parts will not be uploaded to the server until upload is called.

        :param string file_path: Path of file to upload in parts
        """
        with io.open(file_path, mode='rb') as file_object:
            file_object.seek(0, io.SEEK_END)
            end = file_object.tell()
            file_object.seek(0, io.SEEK_SET)
            offset = 0
            while file_object.tell() < end:
                remaining = end - offset
                self.add_part_from_file(file_path, offset=offset, size=self.part_size if remaining > self.part_size else remaining)
                offset += self.part_size
                file_object.seek(offset, io.SEEK_SET)

    def add_part_from_file(self,
                           file_path,
                           offset=0,
                           size=None,
                           part_hash=None):
        """
        Add a part to internal list of parts to upload.  The part will not be uploaded to the server until upload is called.

        :param str file_path: Path to file
        :param offset: Offset of part in file
        :param size: Size of the part in bytes
        :param part_hash: Base64 encoded MD5 hash of the part
        """
        if size is None:
            size = stat(file_path).st_size

        self.manifest["parts"].append({"file_path": file_path,
                                       "offset": offset,
                                       "size": size,
                                       "hash": part_hash})

    def abort(self, **kwargs):
        """
        Abort the multipart upload

        :param str upload_id (optional): The upload id to abort. If not provided, will attempt to abort the upload created internally by new_upload.

        :param str opc_client_request_id: (optional)
            The client request ID for tracing
        """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()
            if 'upload_id' in kwargs:
                self.manifest["uploadId"] = kwargs['upload_id']
                kwargs.pop('upload_id')

            if not self.manifest["uploadId"]:
                raise ValueError("Cannot abort without an upload id.")

            self.object_storage_client.abort_multipart_upload(self.manifest["namespace"],
                                                              self.manifest["bucketName"],
                                                              self.manifest["objectName"],
                                                              self.manifest["uploadId"],
                                                              **kwargs)
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def resume(self, **kwargs):
        """
       Resume uploading a multipart object to Object Storage

       :param str upload_id (optional): The upload id to resume. If not provided, will attempt to resume the upload created internally by new_upload.

       :param function progress_callback (optional):
            Callback function to receive the number of bytes uploaded since
            the last call to the callback function.

       :param str opc_client_request_id: (optional)
            The client request ID for tracing
       """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()
            if 'upload_id' in kwargs:
                self.manifest["uploadId"] = kwargs['upload_id']
                kwargs.pop('upload_id')

            # Verify that the upload id is valid
            if self.manifest["uploadId"] is None:
                raise ValueError("Cannot resume without an upload id.")

            upload_kwargs = {}
            if 'progress_callback' in kwargs:
                upload_kwargs['progress_callback'] = kwargs['progress_callback']
                kwargs.pop('progress_callback')

            # Get parts details from object storage to see which parts didn't complete
            has_next_page = True
            while has_next_page:
                response = self.object_storage_client.list_multipart_upload_parts(self.manifest["namespace"],
                                                                                  self.manifest["bucketName"],
                                                                                  self.manifest["objectName"],
                                                                                  self.manifest["uploadId"],
                                                                                  **kwargs)
                # Update manifest with information from object storage
                parts = self.manifest["parts"]
                for part in response.data:
                    part_index = part.part_number - 1
                    if -1 < part_index < len(parts):
                        manifest_part = parts[part_index]
                        if manifest_part["size"] != part.size:
                            raise ValueError('Cannot resume upload with different part size. Parts were uploaded with a part size of {} MiB'.format(part.size / MEBIBYTE))
                        manifest_part["etag"] = part.etag
                        manifest_part["opc_md5"] = part.md5
                    elif part_index >= len(parts):
                        raise ValueError('There are more parts on the server than parts to resume, please check the upload ID.')
                has_next_page = response.has_next_page
                kwargs['page'] = response.next_page

            # Upload parts that are missing or incomplete
            self.upload(**upload_kwargs)
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def new_upload(self, **kwargs):
        """
        Create a new multipart upload in Object Storage and add the upload
        id to the manifest

        :param str opc_client_request_id: (optional)
            The client request ID for tracing.
        """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            if self.manifest['uploadId']:
                raise RuntimeError('Cannot call new_upload again once an upload has already been created.')

            request = models.CreateMultipartUploadDetails()
            request.object = self.manifest["objectName"]
            if self.content_type:
                request.content_type = self.content_type
            if self.content_language:
                request.content_language = self.content_language
            if self.content_encoding:
                request.content_encoding = self.content_encoding
            if self.metadata:
                # TODO: look into moving this into codegen for create_multipart_upload like it is for put_object
                processed_metadata = {}
                for key, value in six.iteritems(self.metadata):
                    if not key.startswith('opc-meta-'):
                        processed_metadata["opc-meta-" + key] = value
                    else:
                        processed_metadata[key] = value
                self.metadata = processed_metadata

                request.metadata = self.metadata

            client_request_id = None
            if 'opc_client_request_id' in kwargs:
                client_request_id = kwargs['opc_client_request_id']

            kwargs = {}
            if client_request_id:
                kwargs['opc_client_request_id'] = client_request_id
            if self.if_match:
                kwargs['if_match'] = self.if_match
            if self.if_none_match:
                kwargs['if_none_match'] = self.if_none_match

            # pass on SSE-C values (if any)
            kwargs.update(self.ssec_params)

            response = self.object_storage_client.create_multipart_upload(self.manifest["namespace"],
                                                                          self.manifest["bucketName"],
                                                                          request,
                                                                          **kwargs)

            self.manifest["uploadId"] = response.data.upload_id
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def _upload_part(self, part_num, part, **kwargs):
        """
        Upload a part to Object Storage

        :param int part_num: Sequence number for where this part belongs in the
                             multipart object.
        :param dict part: Part dictionary containing the following keys: "file_path" (str), "hash" (str), "offset" (int), and "size" (int)
        :param str opc_client_request_id: (optional)
            The client request ID for tracing.
        :param function progress_callback (optional):
            Callback function to receive the number of bytes uploaded since
            the last call to the callback function.
        """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            new_kwargs = {'content_md5': part["hash"]}
            if 'opc_client_request_id' in kwargs:
                new_kwargs['opc_client_request_id'] = kwargs['opc_client_request_id']

            # supply SSE-C key (if any) information to upload-part
            new_kwargs.update(self.ssec_params)

            # TODO: Calculate the hash without needing to read the file chunk twice.
            # Calculate the hash before uploading.  The hash will be used
            # to determine if the part needs to be uploaded.  It will also
            # be used to determine if there is a conflict between parts that
            # have previously been uploaded.
            if part["hash"] is None:
                part["hash"] = self.calculate_md5(part["file_path"], part["offset"], part["size"])

            retry_strategy = None
            if 'retry_strategy' in kwargs:
                retry_strategy = kwargs['retry_strategy']

            if "opc_md5" not in part:
                # Disable the retry_strategy to work around data corruption issue temporarily
                retry_strategy = None

                if retry_strategy:
                    response = retry_strategy.make_retrying_call(
                        self._upload_part_call,
                        self.object_storage_client,
                        namespace=self.manifest["namespace"],
                        bucket_name=self.manifest["bucketName"],
                        object_name=self.manifest["objectName"],
                        upload_id=self.manifest["uploadId"],
                        part_file_path=part["file_path"],
                        offset=part["offset"],
                        size=part["size"],
                        part_num=part_num,
                        new_kwargs=new_kwargs
                    )
                else:
                    # Disable the retry_strategy to work around data corruption issue temporarily
                    remaining_tries = 1
                    while remaining_tries > 0:
                        try:
                            response = self._upload_part_call(self.object_storage_client,
                                                              namespace=self.manifest["namespace"],
                                                              bucket_name=self.manifest["bucketName"],
                                                              object_name=self.manifest["objectName"],
                                                              upload_id=self.manifest["uploadId"],
                                                              part_file_path=part["file_path"],
                                                              offset=part["offset"],
                                                              size=part["size"],
                                                              part_num=part_num,
                                                              new_kwargs=new_kwargs)
                        except Exception as e:
                            if self._is_exception_retryable(e) and remaining_tries > 1:
                                remaining_tries -= 1
                            else:
                                raise
                        else:
                            break

                if response.status == 200:
                    part["etag"] = response.headers['etag']
                    part["opc_md5"] = str(response.headers['opc-content-md5'])

            if 'progress_callback' in kwargs:
                kwargs['progress_callback'](part["size"])

        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def _upload_stream_part(self, part_num, part_bytes, **kwargs):
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            m = hashlib.md5()
            m.update(part_bytes)

            new_kwargs = {'content_md5': base64.b64encode(m.digest()).decode("utf-8")}
            if 'opc_client_request_id' in kwargs:
                new_kwargs['opc_client_request_id'] = kwargs['opc_client_request_id']

            remaining_tries = self.max_retries
            while remaining_tries > 0:
                try:
                    response = self.object_storage_client.upload_part(
                        self.manifest["namespace"],
                        self.manifest["bucketName"],
                        self.manifest["objectName"],
                        self.manifest["uploadId"],
                        part_num + 1,  # Internally this is 0-based but object storage is 1-based
                        part_bytes,
                        **new_kwargs
                    )
                except Exception as e:
                    if self._is_exception_retryable(e) and remaining_tries > 1:
                        remaining_tries -= 1
                    else:
                        if 'shared_dict' in kwargs:
                            kwargs['shared_dict']['should_continue'] = False
                            kwargs['shared_dict']['exceptions'].put(e)
                        raise
                else:
                    break

            if response.status == 200:
                self.manifest['parts'].append({
                    'etag': response.headers['etag'],
                    'opc_md5': str(response.headers['opc-content-md5']),
                    'part_num': part_num
                })

            if 'progress_callback' in kwargs:
                kwargs['progress_callback'](len(part_bytes))
        finally:
            if 'semaphore' in kwargs:
                kwargs['semaphore'].release()

            # Reset the object storage client timeout regardless of the result of above operations
            self._reset_object_storage_client_timeout()

    def upload(self, **kwargs):
        """
        Upload an object to Object Storage

        :param function progress_callback (optional):
            Callback function to receive the number of bytes uploaded since
            the last call to the callback function.
        :param str opc_client_request_id: (optional)
            The client request ID for tracing.
        """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            if self.manifest["uploadId"] is None:
                raise RuntimeError('Cannot call upload before initializing an upload using new_upload.')

            pool = Pool(processes=self.parallel_process_count)
            pool.map(lambda part_tuple: self._upload_part(part_num=part_tuple[0] + 1, part=part_tuple[1], **kwargs),
                     enumerate(self.manifest["parts"]))
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def upload_stream(self, stream_ref, **kwargs):
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            if self.manifest["uploadId"] is None:
                raise RuntimeError('Cannot call upload before initializing an upload using new_upload.')

            # The pool of work we have available, and the sempahore to gate work into the pool (since just submitting
            # work to the pool doesn't block on the number of processes available to do work in the pool)
            pool = Pool(processes=self.parallel_process_count)
            semaphore = Semaphore(self.parallel_process_count)

            # A dict which will be shared between the threads in our pool (this would not work as-is with processes) but
            # we use threads via multiprocessing.dummy. If we use processes, then a Manager would likely be needed for this.
            #
            # should_continue will only ever be set to False by _upload_stream_part so not too worried if we have multiple
            # writers
            #
            # Queue should be thread safe (though for tracking the exceptions, order doesn't strictly matter)
            shared_dict = {'should_continue': True, 'exceptions': Queue()}

            part_counter = 0

            apply_async_kwargs = kwargs.copy()
            apply_async_kwargs['semaphore'] = semaphore
            apply_async_kwargs['shared_dict'] = shared_dict

            # We pull data from the stream until there is no more
            keep_reading = True
            while keep_reading:
                if six.PY3 and hasattr(stream_ref, 'buffer'):
                    read_bytes = stream_ref.buffer.read(self.part_size)
                else:
                    read_bytes = stream_ref.read(self.part_size)

                semaphore.acquire()

                if len(read_bytes) != 0:
                    pool.apply_async(self._upload_stream_part, (part_counter, read_bytes), apply_async_kwargs)
                    part_counter += 1

                keep_reading = (len(read_bytes) == self.part_size) and shared_dict['should_continue']

            # If we're here we've either sent off all the work we needed to (and so are waiting on remaining bits to finish)
            # or we terminated early because of an exception in one of our uploads. In either case, close off the pool to
            # any more work and let the remaining work finish gracefully
            pool.close()
            pool.join()

            # If we had at least one exception then throw out an error to indicate failure
            if not shared_dict['exceptions'].empty():
                raise MultipartUploadError(error_causes_queue=shared_dict['exceptions'])

            # Because we processed in parallel, the parts in the manifest may be out of order. Re-order them based on the part number
            # because commit assumes that they are ordered
            self.manifest['parts'].sort(key=lambda part: part['part_num'])
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    def commit(self, **kwargs):
        """
        Commit the multipart upload.

        :return: A Response object with data of type None
        :rtype: None
        """
        try:
            # Set the upload manager timeout on object storage client
            self._set_multipart_upload_timeout()

            if self.manifest["uploadId"] is None:
                raise RuntimeError('Cannot call commit before initializing an upload using new_upload or resuming an upload using resume.')

            commit_details = models.CommitMultipartUploadDetails()

            # Determine which parts to commit and which parts to exclude.
            parts_to_commit = []
            parts_to_exclude = []
            for partNum, part in enumerate(self.manifest["parts"]):
                detail = models.CommitMultipartUploadPartDetails()
                detail.part_num = partNum + 1
                if "etag" in part:
                    detail.etag = part["etag"]
                    parts_to_commit.append(detail)
                else:
                    parts_to_exclude.append(partNum + 1)

            commit_details.parts_to_commit = parts_to_commit
            commit_details.parts_to_exclude = parts_to_exclude

            # Commit the multipart upload
            response = self.object_storage_client.commit_multipart_upload(self.manifest["namespace"],
                                                                          self.manifest["bucketName"],
                                                                          self.manifest["objectName"],
                                                                          self.manifest["uploadId"],
                                                                          commit_details,
                                                                          **kwargs)

            return response
        # Reset the object storage client timeout regardless of the result of above operations
        finally:
            self._reset_object_storage_client_timeout()

    # The following two methods are called at the beginning and at the end of every method for upload manager
    # This is not the best way to go around the timeout issue, but it is a workaround for now
    def _set_multipart_upload_timeout(self):
        self.object_storage_client.base_client.timeout = MULTIPART_UPLOAD_TIMEOUT

    def _reset_object_storage_client_timeout(self):
        self.object_storage_client.base_client.timeout = self._object_storage_client_timeout



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreateSddcDetails(object):
    """
    Details of the SDDC.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new CreateSddcDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param compute_availability_domain:
            The value to assign to the compute_availability_domain property of this CreateSddcDetails.
        :type compute_availability_domain: str

        :param display_name:
            The value to assign to the display_name property of this CreateSddcDetails.
        :type display_name: str

        :param vmware_software_version:
            The value to assign to the vmware_software_version property of this CreateSddcDetails.
        :type vmware_software_version: str

        :param compartment_id:
            The value to assign to the compartment_id property of this CreateSddcDetails.
        :type compartment_id: str

        :param instance_display_name_prefix:
            The value to assign to the instance_display_name_prefix property of this CreateSddcDetails.
        :type instance_display_name_prefix: str

        :param esxi_hosts_count:
            The value to assign to the esxi_hosts_count property of this CreateSddcDetails.
        :type esxi_hosts_count: int

        :param ssh_authorized_keys:
            The value to assign to the ssh_authorized_keys property of this CreateSddcDetails.
        :type ssh_authorized_keys: str

        :param workload_network_cidr:
            The value to assign to the workload_network_cidr property of this CreateSddcDetails.
        :type workload_network_cidr: str

        :param provisioning_subnet_id:
            The value to assign to the provisioning_subnet_id property of this CreateSddcDetails.
        :type provisioning_subnet_id: str

        :param vsphere_vlan_id:
            The value to assign to the vsphere_vlan_id property of this CreateSddcDetails.
        :type vsphere_vlan_id: str

        :param vmotion_vlan_id:
            The value to assign to the vmotion_vlan_id property of this CreateSddcDetails.
        :type vmotion_vlan_id: str

        :param vsan_vlan_id:
            The value to assign to the vsan_vlan_id property of this CreateSddcDetails.
        :type vsan_vlan_id: str

        :param nsx_v_tep_vlan_id:
            The value to assign to the nsx_v_tep_vlan_id property of this CreateSddcDetails.
        :type nsx_v_tep_vlan_id: str

        :param nsx_edge_v_tep_vlan_id:
            The value to assign to the nsx_edge_v_tep_vlan_id property of this CreateSddcDetails.
        :type nsx_edge_v_tep_vlan_id: str

        :param nsx_edge_uplink1_vlan_id:
            The value to assign to the nsx_edge_uplink1_vlan_id property of this CreateSddcDetails.
        :type nsx_edge_uplink1_vlan_id: str

        :param nsx_edge_uplink2_vlan_id:
            The value to assign to the nsx_edge_uplink2_vlan_id property of this CreateSddcDetails.
        :type nsx_edge_uplink2_vlan_id: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreateSddcDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this CreateSddcDetails.
        :type defined_tags: dict(str, dict(str, object))

        """
        self.swagger_types = {
            'compute_availability_domain': 'str',
            'display_name': 'str',
            'vmware_software_version': 'str',
            'compartment_id': 'str',
            'instance_display_name_prefix': 'str',
            'esxi_hosts_count': 'int',
            'ssh_authorized_keys': 'str',
            'workload_network_cidr': 'str',
            'provisioning_subnet_id': 'str',
            'vsphere_vlan_id': 'str',
            'vmotion_vlan_id': 'str',
            'vsan_vlan_id': 'str',
            'nsx_v_tep_vlan_id': 'str',
            'nsx_edge_v_tep_vlan_id': 'str',
            'nsx_edge_uplink1_vlan_id': 'str',
            'nsx_edge_uplink2_vlan_id': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))'
        }

        self.attribute_map = {
            'compute_availability_domain': 'computeAvailabilityDomain',
            'display_name': 'displayName',
            'vmware_software_version': 'vmwareSoftwareVersion',
            'compartment_id': 'compartmentId',
            'instance_display_name_prefix': 'instanceDisplayNamePrefix',
            'esxi_hosts_count': 'esxiHostsCount',
            'ssh_authorized_keys': 'sshAuthorizedKeys',
            'workload_network_cidr': 'workloadNetworkCidr',
            'provisioning_subnet_id': 'provisioningSubnetId',
            'vsphere_vlan_id': 'vsphereVlanId',
            'vmotion_vlan_id': 'vmotionVlanId',
            'vsan_vlan_id': 'vsanVlanId',
            'nsx_v_tep_vlan_id': 'nsxVTepVlanId',
            'nsx_edge_v_tep_vlan_id': 'nsxEdgeVTepVlanId',
            'nsx_edge_uplink1_vlan_id': 'nsxEdgeUplink1VlanId',
            'nsx_edge_uplink2_vlan_id': 'nsxEdgeUplink2VlanId',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags'
        }

        self._compute_availability_domain = None
        self._display_name = None
        self._vmware_software_version = None
        self._compartment_id = None
        self._instance_display_name_prefix = None
        self._esxi_hosts_count = None
        self._ssh_authorized_keys = None
        self._workload_network_cidr = None
        self._provisioning_subnet_id = None
        self._vsphere_vlan_id = None
        self._vmotion_vlan_id = None
        self._vsan_vlan_id = None
        self._nsx_v_tep_vlan_id = None
        self._nsx_edge_v_tep_vlan_id = None
        self._nsx_edge_uplink1_vlan_id = None
        self._nsx_edge_uplink2_vlan_id = None
        self._freeform_tags = None
        self._defined_tags = None

    @property
    def compute_availability_domain(self):
        """
        **[Required]** Gets the compute_availability_domain of this CreateSddcDetails.
        The availability domain to create the SDDC's ESXi hosts in.


        :return: The compute_availability_domain of this CreateSddcDetails.
        :rtype: str
        """
        return self._compute_availability_domain

    @compute_availability_domain.setter
    def compute_availability_domain(self, compute_availability_domain):
        """
        Sets the compute_availability_domain of this CreateSddcDetails.
        The availability domain to create the SDDC's ESXi hosts in.


        :param compute_availability_domain: The compute_availability_domain of this CreateSddcDetails.
        :type: str
        """
        self._compute_availability_domain = compute_availability_domain

    @property
    def display_name(self):
        """
        Gets the display_name of this CreateSddcDetails.
        A descriptive name for the SDDC. It must be unique, start with a letter, and contain only letters, digits,
        whitespaces, dashes and underscores.
        Avoid entering confidential information.


        :return: The display_name of this CreateSddcDetails.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this CreateSddcDetails.
        A descriptive name for the SDDC. It must be unique, start with a letter, and contain only letters, digits,
        whitespaces, dashes and underscores.
        Avoid entering confidential information.


        :param display_name: The display_name of this CreateSddcDetails.
        :type: str
        """
        self._display_name = display_name

    @property
    def vmware_software_version(self):
        """
        **[Required]** Gets the vmware_software_version of this CreateSddcDetails.
        The VMware software bundle to install on the ESXi hosts in the SDDC. To get a
        list of the available versions, use
        :func:`
        _list_supported_vmware_software_versions`.


        :return: The vmware_software_version of this CreateSddcDetails.
        :rtype: str
        """
        return self._vmware_software_version

    @vmware_software_version.setter
    def vmware_software_version(self, vmware_software_version):
        """
        Sets the vmware_software_version of this CreateSddcDetails.
        The VMware software bundle to install on the ESXi hosts in the SDDC. To get a
        list of the available versions, use
        :func:`
        _list_supported_vmware_software_versions`.


        :param vmware_software_version: The vmware_software_version of this CreateSddcDetails.
        :type: str
        """
        self._vmware_software_version = vmware_software_version

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this CreateSddcDetails.
        The `OCID`__ of the compartment to contain the SDDC.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The compartment_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this CreateSddcDetails.
        The `OCID`__ of the compartment to contain the SDDC.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param compartment_id: The compartment_id of this CreateSddcDetails.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def instance_display_name_prefix(self):
        """
        Gets the instance_display_name_prefix of this CreateSddcDetails.
        A prefix used in the name of each ESXi host and Compute instance in the SDDC.
        If this isn't set, the SDDC's `displayName` is used as the prefix.

        For example, if the value is `mySDDC`, the ESXi hosts are named `mySDDC-1`,
        `mySDDC-2`, and so on.


        :return: The instance_display_name_prefix of this CreateSddcDetails.
        :rtype: str
        """
        return self._instance_display_name_prefix

    @instance_display_name_prefix.setter
    def instance_display_name_prefix(self, instance_display_name_prefix):
        """
        Sets the instance_display_name_prefix of this CreateSddcDetails.
        A prefix used in the name of each ESXi host and Compute instance in the SDDC.
        If this isn't set, the SDDC's `displayName` is used as the prefix.

        For example, if the value is `mySDDC`, the ESXi hosts are named `mySDDC-1`,
        `mySDDC-2`, and so on.


        :param instance_display_name_prefix: The instance_display_name_prefix of this CreateSddcDetails.
        :type: str
        """
        self._instance_display_name_prefix = instance_display_name_prefix

    @property
    def esxi_hosts_count(self):
        """
        **[Required]** Gets the esxi_hosts_count of this CreateSddcDetails.
        The number of ESXi hosts to create in the SDDC. You can add more hosts later
        (see :func:`create_esxi_host`).

        **Note:** If you later delete EXSi hosts from the SDDC to total less than 3,
        you are still billed for the 3 minimum recommended EXSi hosts. Also,
        you cannot add more VMware workloads to the SDDC until it again has at least
        3 ESXi hosts.


        :return: The esxi_hosts_count of this CreateSddcDetails.
        :rtype: int
        """
        return self._esxi_hosts_count

    @esxi_hosts_count.setter
    def esxi_hosts_count(self, esxi_hosts_count):
        """
        Sets the esxi_hosts_count of this CreateSddcDetails.
        The number of ESXi hosts to create in the SDDC. You can add more hosts later
        (see :func:`create_esxi_host`).

        **Note:** If you later delete EXSi hosts from the SDDC to total less than 3,
        you are still billed for the 3 minimum recommended EXSi hosts. Also,
        you cannot add more VMware workloads to the SDDC until it again has at least
        3 ESXi hosts.


        :param esxi_hosts_count: The esxi_hosts_count of this CreateSddcDetails.
        :type: int
        """
        self._esxi_hosts_count = esxi_hosts_count

    @property
    def ssh_authorized_keys(self):
        """
        **[Required]** Gets the ssh_authorized_keys of this CreateSddcDetails.
        One or more public SSH keys to be included in the `~/.ssh/authorized_keys` file for
        the default user on each ESXi host. Use a newline character to separate multiple keys.
        The SSH keys must be in the format required for the `authorized_keys` file


        :return: The ssh_authorized_keys of this CreateSddcDetails.
        :rtype: str
        """
        return self._ssh_authorized_keys

    @ssh_authorized_keys.setter
    def ssh_authorized_keys(self, ssh_authorized_keys):
        """
        Sets the ssh_authorized_keys of this CreateSddcDetails.
        One or more public SSH keys to be included in the `~/.ssh/authorized_keys` file for
        the default user on each ESXi host. Use a newline character to separate multiple keys.
        The SSH keys must be in the format required for the `authorized_keys` file


        :param ssh_authorized_keys: The ssh_authorized_keys of this CreateSddcDetails.
        :type: str
        """
        self._ssh_authorized_keys = ssh_authorized_keys

    @property
    def workload_network_cidr(self):
        """
        Gets the workload_network_cidr of this CreateSddcDetails.
        The CIDR block for the IP addresses that VMware VMs in the SDDC use to run application
        workloads.


        :return: The workload_network_cidr of this CreateSddcDetails.
        :rtype: str
        """
        return self._workload_network_cidr

    @workload_network_cidr.setter
    def workload_network_cidr(self, workload_network_cidr):
        """
        Sets the workload_network_cidr of this CreateSddcDetails.
        The CIDR block for the IP addresses that VMware VMs in the SDDC use to run application
        workloads.


        :param workload_network_cidr: The workload_network_cidr of this CreateSddcDetails.
        :type: str
        """
        self._workload_network_cidr = workload_network_cidr

    @property
    def provisioning_subnet_id(self):
        """
        **[Required]** Gets the provisioning_subnet_id of this CreateSddcDetails.
        The `OCID`__ of the management subnet to use
        for provisioning the SDDC.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The provisioning_subnet_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._provisioning_subnet_id

    @provisioning_subnet_id.setter
    def provisioning_subnet_id(self, provisioning_subnet_id):
        """
        Sets the provisioning_subnet_id of this CreateSddcDetails.
        The `OCID`__ of the management subnet to use
        for provisioning the SDDC.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param provisioning_subnet_id: The provisioning_subnet_id of this CreateSddcDetails.
        :type: str
        """
        self._provisioning_subnet_id = provisioning_subnet_id

    @property
    def vsphere_vlan_id(self):
        """
        **[Required]** Gets the vsphere_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vSphere
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The vsphere_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._vsphere_vlan_id

    @vsphere_vlan_id.setter
    def vsphere_vlan_id(self, vsphere_vlan_id):
        """
        Sets the vsphere_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vSphere
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param vsphere_vlan_id: The vsphere_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._vsphere_vlan_id = vsphere_vlan_id

    @property
    def vmotion_vlan_id(self):
        """
        **[Required]** Gets the vmotion_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vMotion
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The vmotion_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._vmotion_vlan_id

    @vmotion_vlan_id.setter
    def vmotion_vlan_id(self, vmotion_vlan_id):
        """
        Sets the vmotion_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vMotion
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param vmotion_vlan_id: The vmotion_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._vmotion_vlan_id = vmotion_vlan_id

    @property
    def vsan_vlan_id(self):
        """
        **[Required]** Gets the vsan_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vSAN
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The vsan_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._vsan_vlan_id

    @vsan_vlan_id.setter
    def vsan_vlan_id(self, vsan_vlan_id):
        """
        Sets the vsan_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the vSAN
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param vsan_vlan_id: The vsan_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._vsan_vlan_id = vsan_vlan_id

    @property
    def nsx_v_tep_vlan_id(self):
        """
        **[Required]** Gets the nsx_v_tep_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX VTEP
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The nsx_v_tep_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._nsx_v_tep_vlan_id

    @nsx_v_tep_vlan_id.setter
    def nsx_v_tep_vlan_id(self, nsx_v_tep_vlan_id):
        """
        Sets the nsx_v_tep_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX VTEP
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param nsx_v_tep_vlan_id: The nsx_v_tep_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._nsx_v_tep_vlan_id = nsx_v_tep_vlan_id

    @property
    def nsx_edge_v_tep_vlan_id(self):
        """
        **[Required]** Gets the nsx_edge_v_tep_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge VTEP
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The nsx_edge_v_tep_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._nsx_edge_v_tep_vlan_id

    @nsx_edge_v_tep_vlan_id.setter
    def nsx_edge_v_tep_vlan_id(self, nsx_edge_v_tep_vlan_id):
        """
        Sets the nsx_edge_v_tep_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge VTEP
        component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param nsx_edge_v_tep_vlan_id: The nsx_edge_v_tep_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._nsx_edge_v_tep_vlan_id = nsx_edge_v_tep_vlan_id

    @property
    def nsx_edge_uplink1_vlan_id(self):
        """
        **[Required]** Gets the nsx_edge_uplink1_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge
        Uplink 1 component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The nsx_edge_uplink1_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._nsx_edge_uplink1_vlan_id

    @nsx_edge_uplink1_vlan_id.setter
    def nsx_edge_uplink1_vlan_id(self, nsx_edge_uplink1_vlan_id):
        """
        Sets the nsx_edge_uplink1_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge
        Uplink 1 component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param nsx_edge_uplink1_vlan_id: The nsx_edge_uplink1_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._nsx_edge_uplink1_vlan_id = nsx_edge_uplink1_vlan_id

    @property
    def nsx_edge_uplink2_vlan_id(self):
        """
        **[Required]** Gets the nsx_edge_uplink2_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge
        Uplink 2 component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The nsx_edge_uplink2_vlan_id of this CreateSddcDetails.
        :rtype: str
        """
        return self._nsx_edge_uplink2_vlan_id

    @nsx_edge_uplink2_vlan_id.setter
    def nsx_edge_uplink2_vlan_id(self, nsx_edge_uplink2_vlan_id):
        """
        Sets the nsx_edge_uplink2_vlan_id of this CreateSddcDetails.
        The `OCID`__ of the VLAN to use for the NSX Edge
        Uplink 2 component of the VMware environment.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param nsx_edge_uplink2_vlan_id: The nsx_edge_uplink2_vlan_id of this CreateSddcDetails.
        :type: str
        """
        self._nsx_edge_uplink2_vlan_id = nsx_edge_uplink2_vlan_id

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this CreateSddcDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :return: The freeform_tags of this CreateSddcDetails.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this CreateSddcDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :param freeform_tags: The freeform_tags of this CreateSddcDetails.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this CreateSddcDetails.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :return: The defined_tags of this CreateSddcDetails.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this CreateSddcDetails.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm


        :param defined_tags: The defined_tags of this CreateSddcDetails.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class OrchestrationVariable(object):
    """
    The model of a variable for an orchestration resource.
    """

    #: A constant which can be used with the data_type property of a OrchestrationVariable.
    #: This constant has a value of "STRING"
    DATA_TYPE_STRING = "STRING"

    #: A constant which can be used with the data_type property of a OrchestrationVariable.
    #: This constant has a value of "INTEGER"
    DATA_TYPE_INTEGER = "INTEGER"

    def __init__(self, **kwargs):
        """
        Initializes a new OrchestrationVariable object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param name:
            The value to assign to the name property of this OrchestrationVariable.
        :type name: str

        :param default_value:
            The value to assign to the default_value property of this OrchestrationVariable.
        :type default_value: str

        :param description:
            The value to assign to the description property of this OrchestrationVariable.
        :type description: str

        :param data_type:
            The value to assign to the data_type property of this OrchestrationVariable.
            Allowed values for this property are: "STRING", "INTEGER", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type data_type: str

        :param is_mandatory:
            The value to assign to the is_mandatory property of this OrchestrationVariable.
        :type is_mandatory: bool

        :param hint_message:
            The value to assign to the hint_message property of this OrchestrationVariable.
        :type hint_message: str

        """
        self.swagger_types = {
            'name': 'str',
            'default_value': 'str',
            'description': 'str',
            'data_type': 'str',
            'is_mandatory': 'bool',
            'hint_message': 'str'
        }

        self.attribute_map = {
            'name': 'name',
            'default_value': 'defaultValue',
            'description': 'description',
            'data_type': 'dataType',
            'is_mandatory': 'isMandatory',
            'hint_message': 'hintMessage'
        }

        self._name = None
        self._default_value = None
        self._description = None
        self._data_type = None
        self._is_mandatory = None
        self._hint_message = None

    @property
    def name(self):
        """
        Gets the name of this OrchestrationVariable.
        The name of the variable.


        :return: The name of this OrchestrationVariable.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this OrchestrationVariable.
        The name of the variable.


        :param name: The name of this OrchestrationVariable.
        :type: str
        """
        self._name = name

    @property
    def default_value(self):
        """
        Gets the default_value of this OrchestrationVariable.
        The variable's default value.


        :return: The default_value of this OrchestrationVariable.
        :rtype: str
        """
        return self._default_value

    @default_value.setter
    def default_value(self, default_value):
        """
        Sets the default_value of this OrchestrationVariable.
        The variable's default value.


        :param default_value: The default_value of this OrchestrationVariable.
        :type: str
        """
        self._default_value = default_value

    @property
    def description(self):
        """
        Gets the description of this OrchestrationVariable.
        A description of the variable.


        :return: The description of this OrchestrationVariable.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this OrchestrationVariable.
        A description of the variable.


        :param description: The description of this OrchestrationVariable.
        :type: str
        """
        self._description = description

    @property
    def data_type(self):
        """
        Gets the data_type of this OrchestrationVariable.
        The data type of the variable.

        Allowed values for this property are: "STRING", "INTEGER", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The data_type of this OrchestrationVariable.
        :rtype: str
        """
        return self._data_type

    @data_type.setter
    def data_type(self, data_type):
        """
        Sets the data_type of this OrchestrationVariable.
        The data type of the variable.


        :param data_type: The data_type of this OrchestrationVariable.
        :type: str
        """
        allowed_values = ["STRING", "INTEGER"]
        if not value_allowed_none_or_none_sentinel(data_type, allowed_values):
            data_type = 'UNKNOWN_ENUM_VALUE'
        self._data_type = data_type

    @property
    def is_mandatory(self):
        """
        Gets the is_mandatory of this OrchestrationVariable.
        Whether the variable is mandatory.


        :return: The is_mandatory of this OrchestrationVariable.
        :rtype: bool
        """
        return self._is_mandatory

    @is_mandatory.setter
    def is_mandatory(self, is_mandatory):
        """
        Sets the is_mandatory of this OrchestrationVariable.
        Whether the variable is mandatory.


        :param is_mandatory: The is_mandatory of this OrchestrationVariable.
        :type: bool
        """
        self._is_mandatory = is_mandatory

    @property
    def hint_message(self):
        """
        Gets the hint_message of this OrchestrationVariable.
        A brief textual description that helps to explain the variable.


        :return: The hint_message of this OrchestrationVariable.
        :rtype: str
        """
        return self._hint_message

    @hint_message.setter
    def hint_message(self, hint_message):
        """
        Sets the hint_message of this OrchestrationVariable.
        A brief textual description that helps to explain the variable.


        :param hint_message: The hint_message of this OrchestrationVariable.
        :type: str
        """
        self._hint_message = hint_message

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .route_authorization_policy import RouteAuthorizationPolicy
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class AnyOfRouteAuthorizationPolicy(RouteAuthorizationPolicy):
    """
    If authentication has been performed, validate whether the request scope (if any) applies to this route.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new AnyOfRouteAuthorizationPolicy object with values from keyword arguments. The default value of the :py:attr:`~oci.apigateway.models.AnyOfRouteAuthorizationPolicy.type` attribute
        of this class is ``ANY_OF`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param type:
            The value to assign to the type property of this AnyOfRouteAuthorizationPolicy.
            Allowed values for this property are: "ANONYMOUS", "ANY_OF", "AUTHENTICATION_ONLY"
        :type type: str

        :param allowed_scope:
            The value to assign to the allowed_scope property of this AnyOfRouteAuthorizationPolicy.
        :type allowed_scope: list[str]

        """
        self.swagger_types = {
            'type': 'str',
            'allowed_scope': 'list[str]'
        }

        self.attribute_map = {
            'type': 'type',
            'allowed_scope': 'allowedScope'
        }

        self._type = None
        self._allowed_scope = None
        self._type = 'ANY_OF'

    @property
    def allowed_scope(self):
        """
        **[Required]** Gets the allowed_scope of this AnyOfRouteAuthorizationPolicy.
        A user whose scope includes any of these access ranges is allowed on
        this route. Access ranges are case-sensitive.


        :return: The allowed_scope of this AnyOfRouteAuthorizationPolicy.
        :rtype: list[str]
        """
        return self._allowed_scope

    @allowed_scope.setter
    def allowed_scope(self, allowed_scope):
        """
        Sets the allowed_scope of this AnyOfRouteAuthorizationPolicy.
        A user whose scope includes any of these access ranges is allowed on
        this route. Access ranges are case-sensitive.


        :param allowed_scope: The allowed_scope of this AnyOfRouteAuthorizationPolicy.
        :type: list[str]
        """
        self._allowed_scope = allowed_scope

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class StaticPublicKey(object):
    """
    A static public key which is used to verify the JWT signature.
    """

    #: A constant which can be used with the format property of a StaticPublicKey.
    #: This constant has a value of "JSON_WEB_KEY"
    FORMAT_JSON_WEB_KEY = "JSON_WEB_KEY"

    #: A constant which can be used with the format property of a StaticPublicKey.
    #: This constant has a value of "PEM"
    FORMAT_PEM = "PEM"

    def __init__(self, **kwargs):
        """
        Initializes a new StaticPublicKey object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.apigateway.models.JsonWebKey`
        * :class:`~oci.apigateway.models.PemEncodedPublicKey`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param kid:
            The value to assign to the kid property of this StaticPublicKey.
        :type kid: str

        :param format:
            The value to assign to the format property of this StaticPublicKey.
            Allowed values for this property are: "JSON_WEB_KEY", "PEM", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type format: str

        """
        self.swagger_types = {
            'kid': 'str',
            'format': 'str'
        }

        self.attribute_map = {
            'kid': 'kid',
            'format': 'format'
        }

        self._kid = None
        self._format = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['format']

        if type == 'JSON_WEB_KEY':
            return 'JsonWebKey'

        if type == 'PEM':
            return 'PemEncodedPublicKey'
        else:
            return 'StaticPublicKey'

    @property
    def kid(self):
        """
        **[Required]** Gets the kid of this StaticPublicKey.
        A unique key ID. This key will be used to verify the signature of a
        JWT with matching \"kid\".


        :return: The kid of this StaticPublicKey.
        :rtype: str
        """
        return self._kid

    @kid.setter
    def kid(self, kid):
        """
        Sets the kid of this StaticPublicKey.
        A unique key ID. This key will be used to verify the signature of a
        JWT with matching \"kid\".


        :param kid: The kid of this StaticPublicKey.
        :type: str
        """
        self._kid = kid

    @property
    def format(self):
        """
        **[Required]** Gets the format of this StaticPublicKey.
        The format of the public key.

        Allowed values for this property are: "JSON_WEB_KEY", "PEM", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The format of this StaticPublicKey.
        :rtype: str
        """
        return self._format

    @format.setter
    def format(self, format):
        """
        Sets the format of this StaticPublicKey.
        The format of the public key.


        :param format: The format of this StaticPublicKey.
        :type: str
        """
        allowed_values = ["JSON_WEB_KEY", "PEM"]
        if not value_allowed_none_or_none_sentinel(format, allowed_values):
            format = 'UNKNOWN_ENUM_VALUE'
        self._format = format

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ConfigurationField(object):
    """
    Information required to migrate an application. Populated by the service as the source application is introspected
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ConfigurationField object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param name:
            The value to assign to the name property of this ConfigurationField.
        :type name: str

        :param group:
            The value to assign to the group property of this ConfigurationField.
        :type group: str

        :param type:
            The value to assign to the type property of this ConfigurationField.
        :type type: str

        :param value:
            The value to assign to the value property of this ConfigurationField.
        :type value: str

        :param description:
            The value to assign to the description property of this ConfigurationField.
        :type description: str

        :param is_required:
            The value to assign to the is_required property of this ConfigurationField.
        :type is_required: bool

        :param is_mutable:
            The value to assign to the is_mutable property of this ConfigurationField.
        :type is_mutable: bool

        """
        self.swagger_types = {
            'name': 'str',
            'group': 'str',
            'type': 'str',
            'value': 'str',
            'description': 'str',
            'is_required': 'bool',
            'is_mutable': 'bool'
        }

        self.attribute_map = {
            'name': 'name',
            'group': 'group',
            'type': 'type',
            'value': 'value',
            'description': 'description',
            'is_required': 'isRequired',
            'is_mutable': 'isMutable'
        }

        self._name = None
        self._group = None
        self._type = None
        self._value = None
        self._description = None
        self._is_required = None
        self._is_mutable = None

    @property
    def name(self):
        """
        Gets the name of this ConfigurationField.
        The name of the configuration field


        :return: The name of this ConfigurationField.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this ConfigurationField.
        The name of the configuration field


        :param name: The name of this ConfigurationField.
        :type: str
        """
        self._name = name

    @property
    def group(self):
        """
        Gets the group of this ConfigurationField.
        The name of the group to which this field belongs, if any.


        :return: The group of this ConfigurationField.
        :rtype: str
        """
        return self._group

    @group.setter
    def group(self, group):
        """
        Sets the group of this ConfigurationField.
        The name of the group to which this field belongs, if any.


        :param group: The group of this ConfigurationField.
        :type: str
        """
        self._group = group

    @property
    def type(self):
        """
        Gets the type of this ConfigurationField.
        The configuration field type


        :return: The type of this ConfigurationField.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this ConfigurationField.
        The configuration field type


        :param type: The type of this ConfigurationField.
        :type: str
        """
        self._type = type

    @property
    def value(self):
        """
        Gets the value of this ConfigurationField.
        The value of the field


        :return: The value of this ConfigurationField.
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """
        Sets the value of this ConfigurationField.
        The value of the field


        :param value: The value of this ConfigurationField.
        :type: str
        """
        self._value = value

    @property
    def description(self):
        """
        Gets the description of this ConfigurationField.
        Help text to guide the customer in setting the configuration value


        :return: The description of this ConfigurationField.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this ConfigurationField.
        Help text to guide the customer in setting the configuration value


        :param description: The description of this ConfigurationField.
        :type: str
        """
        self._description = description

    @property
    def is_required(self):
        """
        Gets the is_required of this ConfigurationField.
        Indicates whether or not the field is required (defaults to true)


        :return: The is_required of this ConfigurationField.
        :rtype: bool
        """
        return self._is_required

    @is_required.setter
    def is_required(self, is_required):
        """
        Sets the is_required of this ConfigurationField.
        Indicates whether or not the field is required (defaults to true)


        :param is_required: The is_required of this ConfigurationField.
        :type: bool
        """
        self._is_required = is_required

    @property
    def is_mutable(self):
        """
        Gets the is_mutable of this ConfigurationField.
        Indicates whether or not the field may be modified (defaults to true)


        :return: The is_mutable of this ConfigurationField.
        :rtype: bool
        """
        return self._is_mutable

    @is_mutable.setter
    def is_mutable(self, is_mutable):
        """
        Sets the is_mutable of this ConfigurationField.
        Indicates whether or not the field may be modified (defaults to true)


        :param is_mutable: The is_mutable of this ConfigurationField.
        :type: bool
        """
        self._is_mutable = is_mutable

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class DrgRedundancyStatus(object):
    """
    The redundancy status of the DRG. For more information, see
    `Redundancy Remedies`__.

    __ https://docs.cloud.oracle.com/Content/Network/Troubleshoot/drgredundancy.htm
    """

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_AVAILABLE"
    STATUS_NOT_AVAILABLE = "NOT_AVAILABLE"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "REDUNDANT"
    STATUS_REDUNDANT = "REDUNDANT"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_SINGLE_IPSEC"
    STATUS_NOT_REDUNDANT_SINGLE_IPSEC = "NOT_REDUNDANT_SINGLE_IPSEC"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT"
    STATUS_NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT = "NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_MULTIPLE_IPSECS"
    STATUS_NOT_REDUNDANT_MULTIPLE_IPSECS = "NOT_REDUNDANT_MULTIPLE_IPSECS"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS"
    STATUS_NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS = "NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_MIX_CONNECTIONS"
    STATUS_NOT_REDUNDANT_MIX_CONNECTIONS = "NOT_REDUNDANT_MIX_CONNECTIONS"

    #: A constant which can be used with the status property of a DrgRedundancyStatus.
    #: This constant has a value of "NOT_REDUNDANT_NO_CONNECTION"
    STATUS_NOT_REDUNDANT_NO_CONNECTION = "NOT_REDUNDANT_NO_CONNECTION"

    def __init__(self, **kwargs):
        """
        Initializes a new DrgRedundancyStatus object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this DrgRedundancyStatus.
        :type id: str

        :param status:
            The value to assign to the status property of this DrgRedundancyStatus.
            Allowed values for this property are: "NOT_AVAILABLE", "REDUNDANT", "NOT_REDUNDANT_SINGLE_IPSEC", "NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT", "NOT_REDUNDANT_MULTIPLE_IPSECS", "NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS", "NOT_REDUNDANT_MIX_CONNECTIONS", "NOT_REDUNDANT_NO_CONNECTION", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type status: str

        """
        self.swagger_types = {
            'id': 'str',
            'status': 'str'
        }

        self.attribute_map = {
            'id': 'id',
            'status': 'status'
        }

        self._id = None
        self._status = None

    @property
    def id(self):
        """
        Gets the id of this DrgRedundancyStatus.
        The OCID of the DRG.


        :return: The id of this DrgRedundancyStatus.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this DrgRedundancyStatus.
        The OCID of the DRG.


        :param id: The id of this DrgRedundancyStatus.
        :type: str
        """
        self._id = id

    @property
    def status(self):
        """
        Gets the status of this DrgRedundancyStatus.
        The redundancy status of the DRG.

        Allowed values for this property are: "NOT_AVAILABLE", "REDUNDANT", "NOT_REDUNDANT_SINGLE_IPSEC", "NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT", "NOT_REDUNDANT_MULTIPLE_IPSECS", "NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS", "NOT_REDUNDANT_MIX_CONNECTIONS", "NOT_REDUNDANT_NO_CONNECTION", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The status of this DrgRedundancyStatus.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """
        Sets the status of this DrgRedundancyStatus.
        The redundancy status of the DRG.


        :param status: The status of this DrgRedundancyStatus.
        :type: str
        """
        allowed_values = ["NOT_AVAILABLE", "REDUNDANT", "NOT_REDUNDANT_SINGLE_IPSEC", "NOT_REDUNDANT_SINGLE_VIRTUALCIRCUIT", "NOT_REDUNDANT_MULTIPLE_IPSECS", "NOT_REDUNDANT_MULTIPLE_VIRTUALCIRCUITS", "NOT_REDUNDANT_MIX_CONNECTIONS", "NOT_REDUNDANT_NO_CONNECTION"]
        if not value_allowed_none_or_none_sentinel(status, allowed_values):
            status = 'UNKNOWN_ENUM_VALUE'
        self._status = status

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class MetricData(object):
    """
    The set of aggregated data returned for a metric.
    For information about metrics, see `Metrics Overview`__.

    Limits information for returned data follows.

    * Data points: 100,000.
    * Metric streams* within data points: 2,000.
    * Time range returned for 1-hour resolution: 90 days.
    * Time range returned for 5-minute resolution: 30 days.
    * Time range returned for any other resolution: 7 days.

    *A metric stream is an individual set of aggregated data for a metric, typically specific to a single resource.
    Metric streams cannot be aggregated across metric groups.
    A metric group is the combination of a given metric, metric namespace, and tenancy for the purpose of determining limits.
    For more information about metric-related concepts, see `Monitoring Concepts`__.

    __ https://docs.cloud.oracle.com/iaas/Content/Monitoring/Concepts/monitoringoverview.htm#MetricsOverview
    __ https://docs.cloud.oracle.com/iaas/Content/Monitoring/Concepts/monitoringoverview.htm#concepts
    """

    def __init__(self, **kwargs):
        """
        Initializes a new MetricData object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param namespace:
            The value to assign to the namespace property of this MetricData.
        :type namespace: str

        :param resource_group:
            The value to assign to the resource_group property of this MetricData.
        :type resource_group: str

        :param compartment_id:
            The value to assign to the compartment_id property of this MetricData.
        :type compartment_id: str

        :param name:
            The value to assign to the name property of this MetricData.
        :type name: str

        :param dimensions:
            The value to assign to the dimensions property of this MetricData.
        :type dimensions: dict(str, str)

        :param metadata:
            The value to assign to the metadata property of this MetricData.
        :type metadata: dict(str, str)

        :param resolution:
            The value to assign to the resolution property of this MetricData.
        :type resolution: str

        :param aggregated_datapoints:
            The value to assign to the aggregated_datapoints property of this MetricData.
        :type aggregated_datapoints: list[AggregatedDatapoint]

        """
        self.swagger_types = {
            'namespace': 'str',
            'resource_group': 'str',
            'compartment_id': 'str',
            'name': 'str',
            'dimensions': 'dict(str, str)',
            'metadata': 'dict(str, str)',
            'resolution': 'str',
            'aggregated_datapoints': 'list[AggregatedDatapoint]'
        }

        self.attribute_map = {
            'namespace': 'namespace',
            'resource_group': 'resourceGroup',
            'compartment_id': 'compartmentId',
            'name': 'name',
            'dimensions': 'dimensions',
            'metadata': 'metadata',
            'resolution': 'resolution',
            'aggregated_datapoints': 'aggregatedDatapoints'
        }

        self._namespace = None
        self._resource_group = None
        self._compartment_id = None
        self._name = None
        self._dimensions = None
        self._metadata = None
        self._resolution = None
        self._aggregated_datapoints = None

    @property
    def namespace(self):
        """
        **[Required]** Gets the namespace of this MetricData.
        The reference provided in a metric definition to indicate the source service or
        application that emitted the metric.

        Example: `oci_computeagent`


        :return: The namespace of this MetricData.
        :rtype: str
        """
        return self._namespace

    @namespace.setter
    def namespace(self, namespace):
        """
        Sets the namespace of this MetricData.
        The reference provided in a metric definition to indicate the source service or
        application that emitted the metric.

        Example: `oci_computeagent`


        :param namespace: The namespace of this MetricData.
        :type: str
        """
        self._namespace = namespace

    @property
    def resource_group(self):
        """
        Gets the resource_group of this MetricData.
        Resource group provided with the posted metric. A resource group is a custom string that can be used as a filter. Only one resource group can be applied per metric.
        A valid resourceGroup value starts with an alphabetical character and includes only alphanumeric characters, periods (.), underscores (_), hyphens (-), and dollar signs ($).
        Avoid entering confidential information.

        Example: `frontend-fleet`


        :return: The resource_group of this MetricData.
        :rtype: str
        """
        return self._resource_group

    @resource_group.setter
    def resource_group(self, resource_group):
        """
        Sets the resource_group of this MetricData.
        Resource group provided with the posted metric. A resource group is a custom string that can be used as a filter. Only one resource group can be applied per metric.
        A valid resourceGroup value starts with an alphabetical character and includes only alphanumeric characters, periods (.), underscores (_), hyphens (-), and dollar signs ($).
        Avoid entering confidential information.

        Example: `frontend-fleet`


        :param resource_group: The resource_group of this MetricData.
        :type: str
        """
        self._resource_group = resource_group

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this MetricData.
        The `OCID`__ of the compartment containing the
        resources from which the aggregated data was returned.

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm


        :return: The compartment_id of this MetricData.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this MetricData.
        The `OCID`__ of the compartment containing the
        resources from which the aggregated data was returned.

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm


        :param compartment_id: The compartment_id of this MetricData.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def name(self):
        """
        **[Required]** Gets the name of this MetricData.
        The name of the metric.

        Example: `CpuUtilization`


        :return: The name of this MetricData.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this MetricData.
        The name of the metric.

        Example: `CpuUtilization`


        :param name: The name of this MetricData.
        :type: str
        """
        self._name = name

    @property
    def dimensions(self):
        """
        **[Required]** Gets the dimensions of this MetricData.
        Qualifiers provided in the definition of the returned metric.
        Available dimensions vary by metric namespace. Each dimension takes the form of a key-value pair.

        Example: `\"resourceId\": \"ocid1.instance.region1.phx.exampleuniqueID\"`


        :return: The dimensions of this MetricData.
        :rtype: dict(str, str)
        """
        return self._dimensions

    @dimensions.setter
    def dimensions(self, dimensions):
        """
        Sets the dimensions of this MetricData.
        Qualifiers provided in the definition of the returned metric.
        Available dimensions vary by metric namespace. Each dimension takes the form of a key-value pair.

        Example: `\"resourceId\": \"ocid1.instance.region1.phx.exampleuniqueID\"`


        :param dimensions: The dimensions of this MetricData.
        :type: dict(str, str)
        """
        self._dimensions = dimensions

    @property
    def metadata(self):
        """
        Gets the metadata of this MetricData.
        The references provided in a metric definition to indicate extra information about the metric.

        Example: `\"unit\": \"bytes\"`


        :return: The metadata of this MetricData.
        :rtype: dict(str, str)
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """
        Sets the metadata of this MetricData.
        The references provided in a metric definition to indicate extra information about the metric.

        Example: `\"unit\": \"bytes\"`


        :param metadata: The metadata of this MetricData.
        :type: dict(str, str)
        """
        self._metadata = metadata

    @property
    def resolution(self):
        """
        Gets the resolution of this MetricData.
        The time between calculated aggregation windows. Use with the query interval to vary the
        frequency at which aggregated data points are returned. For example, use a query interval of
        5 minutes with a resolution of 1 minute to retrieve five-minute aggregations at a one-minute
        frequency. The resolution must be equal or less than the interval in the query. The default
        resolution is 1m (one minute). Supported values: `1m`-`60m` (also `1h`).

        Example: `5m`


        :return: The resolution of this MetricData.
        :rtype: str
        """
        return self._resolution

    @resolution.setter
    def resolution(self, resolution):
        """
        Sets the resolution of this MetricData.
        The time between calculated aggregation windows. Use with the query interval to vary the
        frequency at which aggregated data points are returned. For example, use a query interval of
        5 minutes with a resolution of 1 minute to retrieve five-minute aggregations at a one-minute
        frequency. The resolution must be equal or less than the interval in the query. The default
        resolution is 1m (one minute). Supported values: `1m`-`60m` (also `1h`).

        Example: `5m`


        :param resolution: The resolution of this MetricData.
        :type: str
        """
        self._resolution = resolution

    @property
    def aggregated_datapoints(self):
        """
        **[Required]** Gets the aggregated_datapoints of this MetricData.
        The list of timestamp-value pairs returned for the specified request. Metric values are rolled up to the start time specified in the request.
        For important limits information related to data points, see MetricData Reference at the top of this page.


        :return: The aggregated_datapoints of this MetricData.
        :rtype: list[AggregatedDatapoint]
        """
        return self._aggregated_datapoints

    @aggregated_datapoints.setter
    def aggregated_datapoints(self, aggregated_datapoints):
        """
        Sets the aggregated_datapoints of this MetricData.
        The list of timestamp-value pairs returned for the specified request. Metric values are rolled up to the start time specified in the request.
        For important limits information related to data points, see MetricData Reference at the top of this page.


        :param aggregated_datapoints: The aggregated_datapoints of this MetricData.
        :type: list[AggregatedDatapoint]
        """
        self._aggregated_datapoints = aggregated_datapoints

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2020, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreatePatchDetails(object):
    """
    Properties used in patch create operations.
    """

    #: A constant which can be used with the patch_type property of a CreatePatchDetails.
    #: This constant has a value of "PUBLISH"
    PATCH_TYPE_PUBLISH = "PUBLISH"

    #: A constant which can be used with the patch_type property of a CreatePatchDetails.
    #: This constant has a value of "UNPUBLISH"
    PATCH_TYPE_UNPUBLISH = "UNPUBLISH"

    def __init__(self, **kwargs):
        """
        Initializes a new CreatePatchDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param key:
            The value to assign to the key property of this CreatePatchDetails.
        :type key: str

        :param model_version:
            The value to assign to the model_version property of this CreatePatchDetails.
        :type model_version: str

        :param name:
            The value to assign to the name property of this CreatePatchDetails.
        :type name: str

        :param description:
            The value to assign to the description property of this CreatePatchDetails.
        :type description: str

        :param object_status:
            The value to assign to the object_status property of this CreatePatchDetails.
        :type object_status: int

        :param identifier:
            The value to assign to the identifier property of this CreatePatchDetails.
        :type identifier: str

        :param patch_type:
            The value to assign to the patch_type property of this CreatePatchDetails.
            Allowed values for this property are: "PUBLISH", "UNPUBLISH"
        :type patch_type: str

        :param object_keys:
            The value to assign to the object_keys property of this CreatePatchDetails.
        :type object_keys: list[str]

        :param registry_metadata:
            The value to assign to the registry_metadata property of this CreatePatchDetails.
        :type registry_metadata: RegistryMetadata

        """
        self.swagger_types = {
            'key': 'str',
            'model_version': 'str',
            'name': 'str',
            'description': 'str',
            'object_status': 'int',
            'identifier': 'str',
            'patch_type': 'str',
            'object_keys': 'list[str]',
            'registry_metadata': 'RegistryMetadata'
        }

        self.attribute_map = {
            'key': 'key',
            'model_version': 'modelVersion',
            'name': 'name',
            'description': 'description',
            'object_status': 'objectStatus',
            'identifier': 'identifier',
            'patch_type': 'patchType',
            'object_keys': 'objectKeys',
            'registry_metadata': 'registryMetadata'
        }

        self._key = None
        self._model_version = None
        self._name = None
        self._description = None
        self._object_status = None
        self._identifier = None
        self._patch_type = None
        self._object_keys = None
        self._registry_metadata = None

    @property
    def key(self):
        """
        Gets the key of this CreatePatchDetails.
        The key of the object.


        :return: The key of this CreatePatchDetails.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Sets the key of this CreatePatchDetails.
        The key of the object.


        :param key: The key of this CreatePatchDetails.
        :type: str
        """
        self._key = key

    @property
    def model_version(self):
        """
        Gets the model_version of this CreatePatchDetails.
        The model version of an object.


        :return: The model_version of this CreatePatchDetails.
        :rtype: str
        """
        return self._model_version

    @model_version.setter
    def model_version(self, model_version):
        """
        Sets the model_version of this CreatePatchDetails.
        The model version of an object.


        :param model_version: The model_version of this CreatePatchDetails.
        :type: str
        """
        self._model_version = model_version

    @property
    def name(self):
        """
        **[Required]** Gets the name of this CreatePatchDetails.
        Free form text without any restriction on permitted characters. Name can have letters, numbers, and special characters. The value can be edited by the user and it is restricted to 1000 characters


        :return: The name of this CreatePatchDetails.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this CreatePatchDetails.
        Free form text without any restriction on permitted characters. Name can have letters, numbers, and special characters. The value can be edited by the user and it is restricted to 1000 characters


        :param name: The name of this CreatePatchDetails.
        :type: str
        """
        self._name = name

    @property
    def description(self):
        """
        Gets the description of this CreatePatchDetails.
        Detailed description for the object.


        :return: The description of this CreatePatchDetails.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this CreatePatchDetails.
        Detailed description for the object.


        :param description: The description of this CreatePatchDetails.
        :type: str
        """
        self._description = description

    @property
    def object_status(self):
        """
        Gets the object_status of this CreatePatchDetails.
        The status of an object that can be set to value 1 for shallow references across objects, other values reserved.


        :return: The object_status of this CreatePatchDetails.
        :rtype: int
        """
        return self._object_status

    @object_status.setter
    def object_status(self, object_status):
        """
        Sets the object_status of this CreatePatchDetails.
        The status of an object that can be set to value 1 for shallow references across objects, other values reserved.


        :param object_status: The object_status of this CreatePatchDetails.
        :type: int
        """
        self._object_status = object_status

    @property
    def identifier(self):
        """
        **[Required]** Gets the identifier of this CreatePatchDetails.
        Value can only contain upper case letters, underscore and numbers. It should begin with upper case letter or underscore. The value can be edited by the user.


        :return: The identifier of this CreatePatchDetails.
        :rtype: str
        """
        return self._identifier

    @identifier.setter
    def identifier(self, identifier):
        """
        Sets the identifier of this CreatePatchDetails.
        Value can only contain upper case letters, underscore and numbers. It should begin with upper case letter or underscore. The value can be edited by the user.


        :param identifier: The identifier of this CreatePatchDetails.
        :type: str
        """
        self._identifier = identifier

    @property
    def patch_type(self):
        """
        **[Required]** Gets the patch_type of this CreatePatchDetails.
        The type of the patch applied or being applied on the application.

        Allowed values for this property are: "PUBLISH", "UNPUBLISH"


        :return: The patch_type of this CreatePatchDetails.
        :rtype: str
        """
        return self._patch_type

    @patch_type.setter
    def patch_type(self, patch_type):
        """
        Sets the patch_type of this CreatePatchDetails.
        The type of the patch applied or being applied on the application.


        :param patch_type: The patch_type of this CreatePatchDetails.
        :type: str
        """
        allowed_values = ["PUBLISH", "UNPUBLISH"]
        if not value_allowed_none_or_none_sentinel(patch_type, allowed_values):
            raise ValueError(
                "Invalid value for `patch_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._patch_type = patch_type

    @property
    def object_keys(self):
        """
        **[Required]** Gets the object_keys of this CreatePatchDetails.
        The array of object keys to publish into application.


        :return: The object_keys of this CreatePatchDetails.
        :rtype: list[str]
        """
        return self._object_keys

    @object_keys.setter
    def object_keys(self, object_keys):
        """
        Sets the object_keys of this CreatePatchDetails.
        The array of object keys to publish into application.


        :param object_keys: The object_keys of this CreatePatchDetails.
        :type: list[str]
        """
        self._object_keys = object_keys

    @property
    def registry_metadata(self):
        """
        Gets the registry_metadata of this CreatePatchDetails.

        :return: The registry_metadata of this CreatePatchDetails.
        :rtype: RegistryMetadata
        """
        return self._registry_metadata

    @registry_metadata.setter
    def registry_metadata(self, registry_metadata):
        """
        Sets the registry_metadata of this CreatePatchDetails.

        :param registry_metadata: The registry_metadata of this CreatePatchDetails.
        :type: RegistryMetadata
        """
        self._registry_metadata = registry_metadata

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
